divert(-1)
#
# J.Rafa 2006
# Allows to use MAYBE as RHS in access map for connecting host addresses
# (Connect: tag or no tag)
# This feature used together with features 'relay_local_from' or/and
# 'relay_mail_from' restricts them to be applied only to hosts with "MAYBE"
# entries. Other hosts aren't affected.
# For any other purpose, "MAYBE" is equal to "OK".
# This improves security if you have to allow some external hosts to relay.
# CAUTION: if you use "MAYBE" in your access map and then remove this feature
# from your sendmail.mc, your access map will contain invalid entries.
# Sendmail will either report errors or reject connections from "MAYBE" hosts.
# To avoid this, if you want to turn this feature off, do not remove it
# completely from your sendmail.mc but replace it with feature
# 'relay_maybe_disabled'. "MAYBE" will be then equal to "OK".
#
ifdef(`RAJ_HACKED_PROTOM4',`define(`_RELAY_MAYBE_',`')',
`ifdef(`_ACCESS_TABLE_',`dnl   good ...

ifdef(`_RELAY_DB_FROM_', `define(`_RELAY_MAIL_FROM_', `1')')dnl
ifdef(`_RELAY_LOCAL_FROM_', `define(`_RELAY_MAIL_FROM_', `1')')dnl
ifdef(`_RELAY_MAIL_FROM_',`dnl   good ...

ifdef(`_DELAY_CHECKS_',
`ifdef(`_NATURAL_CHECK_ORDER_',`define(`_RELAY_MAYBE_',`')',`')',
`define(`_RELAY_MAYBE_',`')'
)
ifdef(`_RELAY_MAYBE_',`dnl',
`errprint(`*** FEATURE(relay_maybe) is incompatible with FEATURE(delay_checks)
    without FEATURE(natural_check_order). Either use natural_check_order or
    do not use delay_checks.
')'
)',

`errprint(`*** FEATURE(relay_maybe) requires FEATURE(relay_local_from) or/and
    FEATURE(relay_mail_from)
')'
)',

`errprint(`*** FEATURE(relay_maybe) requires FEATURE(access_db)
')'
)'
)

ifdef(`_RELAY_MAYBE_',
`divert(0)
VERSIONID(`@(#) relay_maybe.m4, v 2.1, 2006-09-30 J.Rafa')dnl
ifdef(`RAJ_HACKED_PROTOM4',`divert(-1)',
`define(`testRELAY_MAYBE_IS_OK_',`')dnl
dnl definition copied from proto.m4
ifdef(`confRELAY_MSG', `', `define(`confRELAY_MSG',
ifdef(`_USE_AUTH_', `"550 Relaying denied. Proper authentication required."', `"550 Relaying denied"'))')dnl
ifdef(`_RELAY_MAYBE_DISABLED_',`dnl',
`LOCAL_CONFIG
C{Accept}MAYBE')
ifdef(`_NATURAL_CHECK_ORDER_',
`LOCAL_CONFIG
# this is for Local_check_rcpt, to know which ruleset to call
D{RelayMaybeDefined}YES',`dnl')
divert(9)')
')
SActual_check_rcpt
# this is essentially a copy of Basic_check_rcpt with modified second part
# (Relay_ok) - we will check MAYBE there
dnl code copied from proto.m4
# empty address?
R<>			$#error $@ nouser $: "553 User address required"
R$@			$#error $@ nouser $: "553 User address required"
# check for deferred delivery mode
R$*			$: < $&{deliveryMode} > $1
R< d > $*		$@ deferred
R< $* > $*		$: $2

ifdef(`_REQUIRE_QUAL_RCPT_', `dnl
dnl this code checks for user@host where host is not a FQHN.
dnl it is not activated.
dnl notice: code to check for a recipient without a domain name is
dnl available down below; look for the same macro.
dnl this check is done here because the name might be qualified by the
dnl canonicalization.
# require fully qualified domain part?
dnl very simple canonification: make sure the address is in < >
R$+			$: <?> $1
R<?> <$+>		$: <@> <$1>
R<?> $+			$: <@> <$1>
R<@> < postmaster >	$: postmaster
R<@> < $* @ $+ . $+ >	$: < $1 @ $2 . $3 >
dnl prepend daemon_flags
R<@> $*			$: $&{daemon_flags} $| <@> $1
dnl workspace: ${daemon_flags} $| <@> <address>
dnl _r_equire qual.rcpt: ok
R$* r $* $| <@> < $+ @ $+ >	$: < $3 @ $4 >
dnl do not allow these at all or only from local systems?
R$* r $* $| <@> < $* >	$: < ? $&{client_name} > < $3 >
R<?> < $* >		$: <$1>
R<? $=w> < $* >		$: <$1>
R<? $+> <$+>		$#error $@ 5.5.4 $: "553 Fully qualified domain name required"
dnl remove daemon_flags for other cases
R$* $| <@> $*		$: $2', `dnl')

dnl ##################################################################
dnl call subroutines for recipient and relay
dnl possible returns from subroutines:
dnl $#TEMP	temporary failure
dnl $#error	permanent failure (or temporary if from access map)
dnl $#other	stop processing
dnl RELAY	RELAYing allowed
dnl MAYBE	(for relay only) relaying allowed conditionally,
dnl		depending on sender (relay_local_from or relay_mail_from)
dnl other	otherwise
######################################################################
R$*			$: $1 $| @ $>"Rcpt_ok" $1
dnl temporary failure? remove mark @ and remember
R$* $| @ $#TEMP $+	$: $1 $| T $2
dnl error or ok (stop)
R$* $| @ $#$*		$#$2
ifdef(`_PROMISCUOUS_RELAY_', `divert(-1)', `dnl')
R$* $| @ RELAY		$@ RELAY
dnl something else: call check sender (relay)
R$* $| @ $*		$: O $| $>"Actual_Relay_ok" $1
dnl temporary failure: call check sender (relay)
R$* $| T $+		$: T $2 $| $>"Actual_Relay_ok" $1
dnl temporary failure? return that
R$* $| $#TEMP $+	$#error $2
dnl error or ok (stop)
R$* $| $#$*		$#$2
R$* $| RELAY		$@ RELAY
dnl something else: return previous temp failure
R T $+ $| $*		$#error $1
# anything else is bogus
R$*			$#error $@ 5.7.1 $: confRELAY_MSG


######################################################################
### Actual_Relay_ok: is the relay/sender ok?
### it's a copy of Relay_ok with a modification for MAYBE
dnl input: ignored
dnl output: see explanation at call
######################################################################
SActual_Relay_ok
# anything originating locally is ok
# check IP address
R$*			$: $&{client_addr}
R$@			$@ RELAY		originated locally
R0			$@ RELAY		originated locally
R127.0.0.1		$@ RELAY		originated locally
RIPv6:::1		$@ RELAY		originated locally
R$=R $*			$@ RELAY		relayable IP address
ifdef(`_ACCESS_TABLE_', `dnl
R$*			$: $>A <$1> <?> <+ Connect> <$1>
R<RELAY> $* 		$@ RELAY		relayable IP address
ifdef(`_FFR_REJECT_IP_IN_CHECK_RCPT_',`dnl
dnl this will cause rejections in cases like:
dnl Connect:My.Host.Domain	RELAY
dnl Connect:My.Net		REJECT
dnl since in check_relay client_name is checked before client_addr
R<REJECT> $* 		$@ REJECT		rejected IP address')
ifdef(`_ATMPF_', `R<_ATMPF_> $*		$#TEMP $@ 4.3.0 $: "451 Temporary system failure. Please try again later."', `dnl')
# if MAYBE, check sender address
dnl input ignored
R<MAYBE> $*		$: $1 $| $>maybe_relay_ok dummy
R $* $| RELAY		$@ RELAY
R $* $| $#$*		$@ $#$2
# otherwise, continue
R $* $| $*		$: $1
R<$*> <$*>		$: $2', `dnl')
R$*			$: [ $1 ]		put brackets around it...
R$=w			$@ RELAY		... and see if it is local


# check client name: first: did it resolve?
dnl input: ignored
R$*			$: < $&{client_resolve} >
R<TEMP>			$#TEMP $@ 4.4.0 $: "450 Relaying temporarily denied. Cannot resolve PTR record for " $&{client_addr}
R<FORGED>		$#error $@ 5.7.1 $: "550 Relaying denied. IP name possibly forged " $&{client_name}
R<FAIL>			$#error $@ 5.7.1 $: "550 Relaying denied. IP name lookup failed " $&{client_name}
dnl ${client_resolve} should be OK, so go ahead
R$*			$: <@> $&{client_name}
dnl should not be necessary since it has been done for client_addr already
dnl this rule actually may cause a problem if {client_name} resolves to ""
dnl however, this should not happen since the forward lookup should fail
dnl and {client_resolve} should be TEMP or FAIL.
dnl nevertheless, removing the rule doesn't hurt.
dnl R<@>			$@ RELAY
dnl workspace: <@> ${client_name} (not empty)
# pass to name server to make hostname canonical
R<@> $* $=P 		$:<?>  $1 $2
R<@> $+			$:<?>  $[ $1 $]
dnl workspace: <?> ${client_name} (canonified)
R$* .			$1			strip trailing dots
ifdef(`_RELAY_ENTIRE_DOMAIN_', `dnl
R<?> $* $=m		$@ RELAY', `dnl')
R<?> $=w		$@ RELAY
ifdef(`_RELAY_HOSTS_ONLY_',
`R<?> $=R		$@ RELAY
ifdef(`_ACCESS_TABLE_', `dnl
R<?> $*			$: <$(access Connect:$1 $: ? $)> <$1>
R<?> <$*>		$: <$(access $1 $: ? $)> <$1>',`dnl')',
`R<?> $* $=R			$@ RELAY
ifdef(`_ACCESS_TABLE_', `dnl
R<?> $*			$: $>D <$1> <?> <+ Connect> <$1>',`dnl')')
ifdef(`_ACCESS_TABLE_', `dnl
R<RELAY> $*		$@ RELAY
ifdef(`_ATMPF_', `R<$* _ATMPF_> $*		$#TEMP $@ 4.3.0 $: "451 Temporary system failure. Please try again later."', `dnl')
# if MAYBE, check sender address
dnl input ignored
R<MAYBE> $*		$: $1 $| $>maybe_relay_ok dummy
R $* $| RELAY		$@ RELAY
R $* $| $#$*		$@ $#$2
# otherwise, continue
R $* $| $*		$: $1
R<$*> <$*>		$: $2',`dnl')

Smaybe_relay_ok
ifdef(`_RELAY_MAIL_FROM_', `dnl
dnl input: {client_addr} or something "broken"
dnl just throw the input away; we do not need it.
# check whether FROM is allowed to use system as relay
R$*			$: <?> $>CanonAddr $&f
R<?> $+ < @ $+ . >	<?> $1 < @ $2 >		remove trailing dot
ifdef(`_RELAY_LOCAL_FROM_', `dnl
# check whether local FROM is ok
ifdef(`_VIRTUSER_TABLE_',
`ifdef(`_VIRTUSER_ENTIRE_DOMAIN_',
`R<?> $+ <@$* $={VirtHost}>		$@ RELAY',
`R<?> $+ <@$={VirtHost}>		$@ RELAY')')
R<?> $+ < @ $=w >	$@ RELAY		FROM local', `dnl')
ifdef(`_RELAY_DB_FROM_', `dnl
R<?> $+ < @ $+ >	$: <@> $>SearchList <! From> $| <F:$1@$2> ifdef(`_RELAY_DB_FROM_DOMAIN_', ifdef(`_RELAY_HOSTS_ONLY_', `<E:$2>', `<D:$2>')) <>
R<@> <RELAY>		$@ RELAY		RELAY FROM sender ok
ifdef(`_ATMPF_', `R<@> <_ATMPF_>		$#TEMP $@ 4.3.0 $: "451 Temporary system failure. Please try again later."', `dnl')',
`R<?> $+ < @ $+ >	$: <@> $1 < @ $2 >
ifdef(`_RELAY_DB_FROM_DOMAIN_',
`errprint(`*** ERROR: _RELAY_DB_FROM_DOMAIN_ requires _RELAY_DB_FROM_
')',
`dnl')
dnl')', `dnl')
ifdef(`_RELAY_LOCAL_FROM_',
`R<?> $+			$@ RELAY		local user without domain')
dnl notice: the rulesets above do not leave a unique workspace behind.
dnl it does not matter in this case because the following rule ignores
dnl the input. otherwise these rules must "clean up" the workspace.
dnl end of _PROMISCUOUS_RELAY_
ifdef(`_testRELAY_MAYBE_IS_OK_',`divert(9)')
ifdef(`_NATURAL_CHECK_ORDER_',`dnl',
`
SLocal_check_rcpt
R$*			$: $>Actual_check_rcpt $1
# this ruleset must resolve definitely, to stop Basic_check_rcpt from running
dnl this is incompatible with standard delay_checks, since if this rule returns
dnl a positive value (relaying allowed), all other tests (check_mail and
dnl check_relay) will be skipped!
R$#$*			$1
R$*			$@ $#$1')